
public class FindMatrix {
	// This class contains functions that search for a value v in a matrix mat.
	
	// Brute-force search when no assumption on the matrix mat.
	static int findMatrix(Matrix mat, int v) {
		for (int i = 0; i < mat.size; i++)
			for (int j = 0; j < mat.size; j++)
				if (mat.compare(i, j, v) == 0)
					return i * mat.size + j;
		return -1;
	}

	// Dichotomy search when the rows of the matrix mat are increasing. 
	static int findSortedMatrix(Matrix mat, int v) {
		for (int i = 0; i < mat.size; i++) {
			int t = findSortedRow(mat, i, 0, mat.size, v);
			if (t >= 0)
				return t;
		}
		return -1;
	}

	// auxiliary method: search the value v by dichotomy in row i between columns l (included) and r (excluded). 
	static int findSortedRow(Matrix mat, int i, int l, int r, int v) {
		if (l >= r)
			return -1;
		int m = (l + r) / 2;
		int c = mat.compare(i, m, v);
		if (c == 0)
			return i * mat.size + m;
		else if (c > 0)
			return findSortedRow(mat, i, l, m, v);
		else
			return findSortedRow(mat, i, m+1, r, v);
	}

	// Recursive search when the rows and the columns of the matrix mat are increasing.
	static int findDoublySortedMatrix(Matrix mat, int v) {
		return findDoublySortedMatrix(mat, 0, mat.size - 1, v);
	}

	// auxiliary method: search the value v in the matrix mat between rows imin and size-1 and between columns 0 and jmax (all included).
	static int findDoublySortedMatrix(Matrix mat, int imin, int jmax, int v) {
		if (imin >= mat.size || jmax < 0)
			return -1;
		int c = mat.compare(imin, jmax, v);
		if (c == 0)
			return imin * mat.size + jmax;
		else if (c < 0)
			return findDoublySortedMatrix(mat, imin + 1, jmax, v);
		else
			return findDoublySortedMatrix(mat, imin, jmax - 1, v);
	}

	// construct the size x size matrix with entries 2^i*3^j, using only additions.
	static Matrix HammingMatrix(int size) {
		int[][] data = new int[size][size];
		for (int j = 0; j < size; j++) {
			for (int i = 0; i < size; i++) {
				if (i == 0 && j == 0)
					data[0][0] = 1;
				else if (j == 0)
					data[i][j] = data[i - 1][j] + data[i - 1][j];
				else if (i < size - 1)
					data[i][j] = data[i][j - 1] + data[i + 1][j - 1];
				else
					data[i][j] = 2 * data[i - 1][j];
			}
		}
		return new Matrix(data);
	}
}
