
public class Test321 {

	// print table
	static String print(float[] table) {
		String res = "[";
		for (int i = 0; i < table.length; i++) {
			res += table[i];
			if (i != table.length - 1)
				res += ", ";
		}
		return res + "]";
	}

	// test equal tables
	static boolean equal(float[] table1, float[] table2) {
		if (table1.length != table2.length)
			return false;
		for (int i = 0; i < table1.length; i++)
			if (table1[i] != table2[i])
				return false;
		return true;
	}

	// print a tree
	static String print(Tree t, boolean verbose) {
		String res = "[";
		for (Tree child : t.children)
			res += print(child, false);
		res += "]";
		if (verbose)
			res += " with relativePosition = " + t.relativePosition + 
				   ", leftmostPositions = " + print(t.leftmostPositions) + 
				   ", rightmostPositions = " + print(t.rightmostPositions) + 
				   ", minLeftmostPosition = " + t.minLeftmostPosition + 
				   ", maxRightmostPosition = " + t.maxRightmostPosition + "\n";
		return res;
	}

	// print initial conditions
	static String printInitial(Tree t) {
		String res = "tree\n   " + print(t, false) + "\nwith children\n";
		for (Tree child : t.children)
			res += "   " + print(child, true);
		return res;
	}

	// test the method updateShadow
	static void testUpdateShadow(Tree t, float[] leftmostPositions, float[] rightmostPositions, float minLeftmostPosition, float maxRightmostPosition) {
		t.updateShadow();
		assert(equal(t.leftmostPositions, leftmostPositions)) : "\nThe " + printInitial(t) + "has leftmost positions " + print(leftmostPositions) + " and not " + print(t.leftmostPositions) + ".\n";
		assert(equal(t.rightmostPositions, rightmostPositions)) : "\nThe " + printInitial(t) + "has rightmost positions " + print(rightmostPositions) + " and not " + print(t.rightmostPositions) + ".\n";
		assert(t.minLeftmostPosition == minLeftmostPosition) : "\nThe " + printInitial(t) + "has minimal leftmost position " + minLeftmostPosition + " and not " + t.minLeftmostPosition + ".\n";
		assert(t.maxRightmostPosition == maxRightmostPosition) : "\nThe " + printInitial(t) + "has maximal rightmost position " + maxRightmostPosition + " and not " + t.maxRightmostPosition + ".\n";
	}

	public static void main(String[] args) {
		// check -ea option on JVM
		if (!Test321.class.desiredAssertionStatus()) {
			System.err.println("Vous devez activer l'option -ea de la JVM");
			System.err.println("(Run As -> Run configurations -> Arguments -> VM Arguments)");
			System.exit(1);
		}

		// test the method updateShadow on some examples
		System.out.print("Test de updateShadow ... ");
		
		Tree t1 = new Tree(new Tree[] {});
		Tree t2 = new Tree(new Tree[] {new Tree(), new Tree(), new Tree(new Tree[] {new Tree()})});
		Tree t3 = new Tree(new Tree[] {t1, t2});
		Tree t4 = new Tree(new Tree[] {new Tree(), new Tree(new Tree[] {new Tree(), new Tree(), new Tree()}), t3});

		testUpdateShadow(t1, new float[] {0}, new float[] {0}, 0, 0);

		t2.children[0].relativePosition = -1;
		t2.children[1].relativePosition = 0;
		t2.children[2].relativePosition = 1;
		t2.children[2].leftmostPositions = new float[] {0, 0};
		t2.children[2].rightmostPositions = new float[] {0, 0};
		testUpdateShadow(t2, new float[] {0, -1, 1}, new float[] {0, 1, 1}, -1, 1);

		t1.relativePosition = (float) -0.5;
		t2.relativePosition = (float) 0.5;
		testUpdateShadow(t3, new float[] {0, (float) (-0.5), (float) (-0.5), (float) 1.5},
							 new float[] {0, (float) 0.5, (float) 1.5, (float) 1.5},
							 (float) -0.5, (float) 1.5);

		t4.children[0].relativePosition = -2;
		t4.children[1].relativePosition = 0;
		t4.children[1].leftmostPositions = new float[] {0, -1};
		t4.children[1].rightmostPositions = new float[] {0, 1};
		t4.children[2].relativePosition = 2;
		testUpdateShadow(t4, new float[] {0, -2, -1, (float) 1.5, (float) 3.5}, 
							 new float[] {0, 2, (float) 2.5, (float) 3.5, (float) 3.5},
							 -2, (float) 3.5);
		
		System.out.println("[OK]");
	}
}
