
public class Test323 {

	// print table
	static String print(float[] table) {
		String res = "[";
		for (int i = 0; i < table.length; i++) {
			res += table[i];
			if (i != table.length - 1)
				res += ", ";
		}
		return res + "]";
	}

	// print a tree
	static String print(Tree t, boolean verbose) {
		String res = "[";
		for (Tree child : t.children)
			res += print(child, false);
		res += "]";
		if (verbose)
			res += " with leftmostPositions = " + print(t.leftmostPositions) + 
				   ", rightmostPositions = " + print(t.rightmostPositions) +
				   ", minLeftmostPosition = " + t.minLeftmostPosition + 
				   ", maxRightmostPosition = " + t.maxRightmostPosition + "\n";
		return res;
	}

	// print initial conditions
	static String printInitial(Tree t) {
		String res = "tree\n   " + print(t, false) + "\nwith children\n";
		for (Tree child : t.children)
			res += "   " + print(child, true);
		return res;
	}

	// test the method computeChildrenPositions
	static void testComputeChildrenPositions(Tree t, float[] childrenPositions) {
		t.computeChildrenPositions();
		for (int i = 0; i < t.children.length; i++)
			assert(t.children[i].relativePosition == childrenPositions[i]) : "\nThe " + printInitial(t) + "should place its " + i + "th child at abscissa " + childrenPositions[i] + " and not " + t.children[i].relativePosition + ".\n";
	}

	public static void main(String[] args) {
		// check -ea option on JVM
		if (!Test323.class.desiredAssertionStatus()) {
			System.err.println("Vous devez activer l'option -ea de la JVM");
			System.err.println("(Run As -> Run configurations -> Arguments -> VM Arguments)");
			System.exit(1);
		}

		// test the method computeChildrenPositions on some examples
		System.out.print("Test de computeChildrenPositions ... ");
		
		Tree t1 = new Tree(new Tree[] {});
		Tree t2 = new Tree(new Tree[] {new Tree(), new Tree(), new Tree(new Tree[] {new Tree()})});
		Tree t3 = new Tree(new Tree[] {t1, t2});
		Tree t4 = new Tree(new Tree[] {new Tree(), new Tree(new Tree[] {new Tree(), new Tree(), new Tree()}), t3});

		testComputeChildrenPositions(t1, new float[] {0});

		t2.children[2].updateShadow();
		testComputeChildrenPositions(t2, new float[] {-1, 0, 1});
		t2.updateShadow();
		
		testComputeChildrenPositions(t3, new float[] {(float) -0.5, (float) 0.5});
		t3.updateShadow();
		
		t4.children[1].computeChildrenPositions();
		t4.children[1].updateShadow();
		testComputeChildrenPositions(t4, new float[] {(float) -1.75, (float) -0.75, (float) 1.75});
		t4.updateShadow();
		
		System.out.println("[OK]");
	}

}
