
import java.awt.*;
import java.util.Collections;
import java.util.LinkedList;

import javax.swing.JComponent;
import javax.swing.JFrame;

class TreeViewer extends JFrame {

	private static final long serialVersionUID = -7498525833438154949L;
	static int xLocation = 0;

	public TreeViewer(Tree tree) {
		this.setLocation(xLocation, 0);
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		TreeViewerComponent viewer = new TreeViewerComponent(tree); 
		add(viewer);
		pack();
		setVisible(true);
		xLocation += viewer.getWidth();
	}
}

class TreeViewerComponent extends JComponent {

	private static final long serialVersionUID = -7710437354239150390L;
	private Tree tree;
	int radius = 5, margin = 5, xShift = 40, yShift = 100;

	public TreeViewerComponent(Tree tree) {
		assert tree.maxRightmostPosition < 100 && tree.minLeftmostPosition > -100;
		this.tree = tree;
		int width = (int) (xShift * (tree.maxRightmostPosition - tree.minLeftmostPosition)) + 2 * radius + 2 * margin;
		int height =  yShift * (tree.height - 1) + 2 * radius + 2 * margin;
		setPreferredSize(new Dimension(width, height));
	}

	void draw(Tree t, float x, int y, Graphics2D g2) {
		g2.setColor(Color.BLACK);
		g2.fillRect((int) (x * xShift) + margin, (int) (y * yShift) + margin, 2 * radius, 2 * radius);
		for (Tree child : t.children) {
			g2.drawLine((int) (x * xShift) + radius + margin, (int) (y * yShift) + radius + margin, (int) ((x + child.relativePosition) * xShift) + radius + margin, (int) ((y + 1) * yShift) + radius + margin);
			draw(child, x + child.relativePosition, y + 1, g2);
		}
	}
	
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;

		// set the background color
		Dimension d = getSize();
		g2.setBackground(Color.white);
		g2.clearRect(0, 0, d.width, d.height);

		// draw the tree
		draw(tree, -tree.minLeftmostPosition, 0, g2);
	}
}
